@@ -4,6 +4,8 @@
 _XTRACE_NETWORKING_BGPVPN=$(set +o | grep xtrace)
 set -o xtrace
 
+source $NEUTRON_DIR/devstack/lib/l2_agent
+
 if [[ "$1" == "source" ]]; then
     # no-op
     :
@@ -17,7 +19,7 @@ elif [[ "$1" == "stack" && "$2" == "pre-install" ]]; then
         echo "networking-bagpipe: you don't need to set Q_AGENT=bagpipe-openvswitch anymore"
         Q_AGENT=openvswitch``
     fi
-elif [[ "$1" == "stack" && "$2" == "extra" ]]; then
+elif [[ "$1" == "stack" && "$2" == "test-config" ]]; then
     if is_service_enabled tempest; then
         echo_summary "Enabling bgpvpn in $TEMPEST_CONFIG"
         iniset $TEMPEST_CONFIG service_available bgpvpn "True"
@@ -32,9 +34,12 @@ elif [[ "$1" == "stack" && "$2" == "post-config" ]]; then
     fi
     if is_service_enabled q-agt && is_service_enabled b-bgp && [[ "$Q_AGENT" == "openvswitch" ]]; then
         echo_summary "Configuring OVS agent for bagpipe"
-        source $NEUTRON_DIR/devstack/lib/l2_agent
+
         plugin_agent_add_l2_agent_extension bagpipe_bgpvpn
         configure_l2_agent
+        # l2pop and arp_responder are required for bagpipe driver
+        iniset /$Q_PLUGIN_CONF_FILE agent l2_population True
+        iniset /$Q_PLUGIN_CONF_FILE agent arp_responder True
     fi
     if is_service_enabled h-eng;then
         echo_summary "Enabling bgpvpn in $HEAT_CONF"
