@@ -20,9 +20,9 @@ import sqlalchemy as sa
 from sqlalchemy import orm
 from sqlalchemy.orm import exc
 
-from neutron.api.v2 import attributes as attr
 from neutron.db import common_db_mixin
 
+from neutron_lib.db import constants as db_const
 from neutron_lib.db import model_base
 
 from networking_bgpvpn._i18n import _LI
@@ -35,7 +35,8 @@ LOG = log.getLogger(__name__)
 
 class HasProjectNotNullable(model_base.HasProject):
 
-    project_id = sa.Column(sa.String(attr.TENANT_ID_MAX_LEN), index=True,
+    project_id = sa.Column(sa.String(db_const.PROJECT_ID_FIELD_SIZE),
+                           index=True,
                            nullable=False)
 
 
@@ -87,11 +88,11 @@ class BGPVPN(model_base.BASEV2, model_base.HasId, model_base.HasProject):
     route_distinguishers = sa.Column(sa.String(255), nullable=True)
     network_associations = orm.relationship("BGPVPNNetAssociation",
                                             backref="bgpvpn",
-                                            lazy='joined',
+                                            lazy='select',
                                             cascade='all, delete-orphan')
     router_associations = orm.relationship("BGPVPNRouterAssociation",
                                            backref="bgpvpn",
-                                           lazy='joined',
+                                           lazy='select',
                                            cascade='all, delete-orphan')
 
 
@@ -131,6 +132,26 @@ class BGPVPNPluginDb(common_db_mixin.CommonDbMixin):
         }
         return self._fields(res, fields)
 
+    def _list_bgpvpns_result_filter_hook(self, query, filters):
+        values = filters and filters.get('networks', [])
+        if values:
+            query = query.join(BGPVPNNetAssociation)
+            query = query.filter(BGPVPNNetAssociation.network_id.in_(values))
+
+        values = filters and filters.get('routers', [])
+        if values:
+            query = query.join(BGPVPNRouterAssociation)
+            query = query.filter(BGPVPNRouterAssociation.router_id.in_(values))
+
+        return query
+
+    common_db_mixin.CommonDbMixin.register_model_query_hook(
+        BGPVPN,
+        "bgpvpn_filter_by_resource_association",
+        None,
+        None,
+        '_list_bgpvpns_result_filter_hook')
+
     def create_bgpvpn(self, context, bgpvpn):
         rt = utils.rtrd_list2str(bgpvpn['route_targets'])
         i_rt = utils.rtrd_list2str(bgpvpn['import_targets'])
@@ -193,15 +214,6 @@ class BGPVPNPluginDb(common_db_mixin.CommonDbMixin):
             context.session.delete(bgpvpn_db)
         return bgpvpn
 
-    def find_bgpvpns_for_network(self, context, network_id, bgpvpn_type=None):
-        # Note : we could use added backref in the network table
-        query = (context.session.query(BGPVPN).
-                 join(BGPVPNNetAssociation).
-                 filter(BGPVPNNetAssociation.network_id == network_id))
-        if bgpvpn_type is not None:
-            query = query.filter(BGPVPN.type == bgpvpn_type)
-        return [self._make_bgpvpn_dict(bgpvpn) for bgpvpn in query.all()]
-
     def _make_net_assoc_dict(self, net_assoc_db, fields=None):
         res = {'id': net_assoc_db['id'],
                'tenant_id': net_assoc_db['tenant_id'],
@@ -258,13 +270,6 @@ class BGPVPNPluginDb(common_db_mixin.CommonDbMixin):
             context.session.delete(net_assoc_db)
         return net_assoc
 
-    def find_bgpvpns_for_router(self, context, router_id):
-        query = (context.session.query(BGPVPN).
-                 join(BGPVPNRouterAssociation).
-                 filter(BGPVPNRouterAssociation.router_id == router_id))
-
-        return [self._make_bgpvpn_dict(bgpvpn) for bgpvpn in query.all()]
-
     def _make_router_assoc_dict(self, router_assoc_db, fields=None):
         res = {'id': router_assoc_db['id'],
                'tenant_id': router_assoc_db['tenant_id'],
