@@ -13,28 +13,28 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from sqlalchemy.orm import exc
+from sqlalchemy import orm
 from sqlalchemy import sql
 
 from neutron.callbacks import events
 from neutron.callbacks import registry
 from neutron.callbacks import resources
-from neutron.common import constants as const
-from neutron.common import exceptions as n_exc
-from neutron import context as n_context
 from neutron.db import external_net_db
 from neutron.db import l3_db
 from neutron.db import models_v2
 from neutron.debug import debug_agent
-from neutron.extensions import portbindings
-from neutron.i18n import _LE
-from neutron import manager
+
+from neutron_lib.api.definitions import portbindings
+from neutron_lib import constants as const
+from neutron_lib import exceptions as n_exc
 
 from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
 
 from networking_bagpipe.agent.bgpvpn import rpc_client
 
+from networking_bgpvpn._i18n import _LE
+
 from networking_bgpvpn.neutron.db import bgpvpn_db
 from networking_bgpvpn.neutron.extensions import bgpvpn as bgpvpn_ext
 from networking_bgpvpn.neutron.services.common import constants
@@ -52,10 +52,9 @@ class BGPVPNExternalNetAssociation(n_exc.NeutronException):
                 "network to a BGPVPN")
 
 
-def get_network_info_for_port(context, port_id):
+@log_helpers.log_method_call
+def get_network_info_for_port(context, port_id, network_id):
     """Get MAC, IP and Gateway IP addresses informations for a specific port"""
-
-    LOG.debug("get_network_info_for_port() called for port %s" % port_id)
     try:
         with context.session.begin(subtransactions=True):
             net_info = (context.session.
@@ -71,74 +70,90 @@ def get_network_info_for_port(context, port_id):
                         filter(models_v2.Port.id == port_id).one())
 
             (mac_address, ip_address, cidr, gateway_ip) = net_info
-
-            return {'mac_address': mac_address,
-                    'ip_address': ip_address + cidr[cidr.index('/'):],
-                    'gateway_ip': gateway_ip}
-    except exc.NoResultFound:
+    except orm.exc.NoResultFound:
         return
 
+    gateway_mac = (
+        context.session.
+        query(models_v2.Port.mac_address).
+        filter(
+            models_v2.Port.network_id == network_id,
+            (models_v2.Port.device_owner ==
+             const.DEVICE_OWNER_ROUTER_INTF)
+        ).
+        one_or_none()
+    )
+
+    return {'mac_address': mac_address,
+            'ip_address': ip_address + cidr[cidr.index('/'):],
+            'gateway_ip': gateway_ip,
+            'gateway_mac': gateway_mac[0] if gateway_mac else None}
+
+
+def get_gateway_mac(context, network_id):
+    with context.session.begin(subtransactions=True):
+        gateway_mac = (
+            context.session.
+            query(models_v2.Port.mac_address).
+            filter(
+                models_v2.Port.network_id == network_id,
+                (models_v2.Port.device_owner ==
+                 const.DEVICE_OWNER_ROUTER_INTF)
+            ).
+            one_or_none()
+        )
+
+        return gateway_mac[0] if gateway_mac else None
+
 
 def get_network_ports(context, network_id):
-    try:
-        return (context.session.query(models_v2.Port).
-                filter(models_v2.Port.network_id == network_id,
-                       models_v2.Port.admin_state_up == sql.true()).all())
-    except exc.NoResultFound:
-        return
+    # NOTE(tmorin): currents callers don't look at detailed results
+    # but only test if at least one result exist => can be optimized
+    # by returning a count, rather than all port information
+    return (context.session.query(models_v2.Port).
+            filter(models_v2.Port.network_id == network_id,
+                   models_v2.Port.admin_state_up == sql.true()).all())
 
 
 def get_router_ports(context, router_id):
-    try:
-        return (
-            context.session.query(models_v2.Port).
-            filter(
-                models_v2.Port.device_id == router_id,
-                models_v2.Port.device_owner == const.DEVICE_OWNER_ROUTER_INTF
-            ).all()
-        )
-    except exc.NoResultFound:
-        return
+    return (
+        context.session.query(models_v2.Port).
+        filter(
+            models_v2.Port.device_id == router_id,
+            models_v2.Port.device_owner == const.DEVICE_OWNER_ROUTER_INTF
+        ).all()
+    )
 
 
 def get_router_bgpvpn_assocs(context, router_id):
-    try:
-        return (
-            context.session.query(bgpvpn_db.BGPVPNRouterAssociation).
-            filter(
-                bgpvpn_db.BGPVPNRouterAssociation.router_id == router_id
-            ).all()
-        )
-    except exc.NoResultFound:
-        return []
+    return (
+        context.session.query(bgpvpn_db.BGPVPNRouterAssociation).
+        filter(
+            bgpvpn_db.BGPVPNRouterAssociation.router_id == router_id
+        ).all()
+    )
 
 
 def get_network_bgpvpn_assocs(context, net_id):
-    try:
-        return (
-            context.session.query(bgpvpn_db.BGPVPNNetAssociation).
-            filter(
-                bgpvpn_db.BGPVPNNetAssociation.network_id == net_id
-            ).all()
-        )
-    except exc.NoResultFound:
-        return
+    return (
+        context.session.query(bgpvpn_db.BGPVPNNetAssociation).
+        filter(
+            bgpvpn_db.BGPVPNNetAssociation.network_id == net_id
+        ).all()
+    )
 
 
 def get_bgpvpns_of_router_assocs_by_network(context, net_id):
-    try:
-        return (
-            context.session.query(bgpvpn_db.BGPVPN).
-            join(bgpvpn_db.BGPVPN.router_associations).
-            join(bgpvpn_db.BGPVPNRouterAssociation.router).
-            join(l3_db.Router.attached_ports).
-            join(l3_db.RouterPort.port).
-            filter(
-                models_v2.Port.network_id == net_id
-            ).all()
-        )
-    except exc.NoResultFound:
-        return
+    return (
+        context.session.query(bgpvpn_db.BGPVPN).
+        join(bgpvpn_db.BGPVPN.router_associations).
+        join(bgpvpn_db.BGPVPNRouterAssociation.router).
+        join(l3_db.Router.attached_ports).
+        join(l3_db.RouterPort.port).
+        filter(
+            models_v2.Port.network_id == net_id
+        ).all()
+    )
 
 
 def network_is_external(context, net_id):
@@ -146,10 +161,21 @@ def network_is_external(context, net_id):
         context.session.query(external_net_db.ExternalNetwork).filter_by(
             network_id=net_id).one()
         return True
-    except exc.NoResultFound:
+    except orm.exc.NoResultFound:
         return False
 
 
+def _log_callback_processing_exception(resource, event, trigger, kwargs, e):
+    LOG.exception(_LE("Error during notification processing "
+                      "%(resource)s %(event)s, %(trigger)s, "
+                      "%(kwargs)s: %(exc)s"),
+                  {'trigger': trigger,
+                   'resource': resource,
+                   'event': event,
+                   'kwargs': kwargs,
+                   'exc': e})
+
+
 class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
 
     """BGPVPN Service Driver class for BaGPipe"""
@@ -159,30 +185,31 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
 
         self.agent_rpc = rpc_client.BGPVPNAgentNotifyApi()
 
-        registry.subscribe(self.registry_port_updated, resources.PORT,
+        registry.subscribe(self.registry_port_updated,
+                           resources.PORT,
                            events.AFTER_UPDATE)
 
-        # we need to subscribe to before_delete events, because
-        # on after_delete events the port is already removed from the db
-        # and we can't retrieve the binding:host_id information (which
-        # is not passed in the event either)
-        registry.subscribe(self.registry_port_deleted, resources.PORT,
-                           events.BEFORE_DELETE)
+        registry.subscribe(self.registry_port_deleted,
+                           resources.PORT,
+                           events.AFTER_DELETE)
 
-        # REVISIT(tmorin): if/when port ABORT_DELETE events are implemented
-        #  we will have to revisit the issue, so that the action done after
-        #  BEFORE_DELETE is reverted if needed (or a different solution)
-
-        registry.subscribe(self.registry_port_created, resources.PORT,
+        registry.subscribe(self.registry_router_interface_created,
+                           resources.ROUTER_INTERFACE,
                            events.AFTER_CREATE)
 
-    def _format_bgpvpn(self, bgpvpn, network_id):
+        registry.subscribe(self.registry_router_interface_deleted,
+                           resources.ROUTER_INTERFACE,
+                           events.AFTER_DELETE)
+
+    def _format_bgpvpn(self, context, bgpvpn, network_id):
         """JSON-format BGPVPN
 
         BGPVPN, network identifiers, and route targets.
         """
         formatted_bgpvpn = {'id': bgpvpn['id'],
-                            'network_id': network_id}
+                            'network_id': network_id,
+                            'gateway_mac': get_gateway_mac(context,
+                                                           network_id)}
         formatted_bgpvpn.update(
             self._format_bgpvpn_network_route_targets([bgpvpn]))
 
@@ -242,6 +269,17 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
 
         return bgpvpn_rts
 
+    def _bgpvpns_for_network(self, context, network_id):
+        return (
+            self.bgpvpn_db.get_bgpvpns(
+                context,
+                filters={
+                    'networks': [network_id],
+                },
+            ) or self.retrieve_bgpvpns_of_router_assocs_by_network(context,
+                                                                   network_id)
+        )
+
     def _retrieve_bgpvpn_network_info_for_port(self, context, port):
         """Retrieve BGP VPN network informations for a specific port
 
@@ -250,6 +288,7 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
             'mac_address': '00:00:de:ad:be:ef',
             'ip_address': '10.0.0.2',
             'gateway_ip': '10.0.0.1',
+            'gateway_mac': 'aa:bb:cc:dd:ee:ff', # if a router interface exists
             'l3vpn' : {
                 'import_rt': ['12345:1', '12345:2', '12345:3'],
                 'export_rt': ['12345:1', '12345:2', '12345:4']
@@ -261,12 +300,7 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
 
         bgpvpn_network_info = {}
 
-        # Check if port is connected on a BGP VPN network
-        bgpvpns = (
-            self.bgpvpn_db.find_bgpvpns_for_network(context, network_id)
-            or self.retrieve_bgpvpns_of_router_assocs_by_network(context,
-                                                                 network_id)
-        )
+        bgpvpns = self._bgpvpns_for_network(context, network_id)
 
         # NOTE(tmorin): We currently need to send 'network_id', 'mac_address',
         #   'ip_address', 'gateway_ip' to the agent, even in the absence of
@@ -283,7 +317,7 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
         bgpvpn_network_info.update(bgpvpn_rts)
 
         LOG.debug("Getting port %s network details" % port_id)
-        network_info = get_network_info_for_port(context, port_id)
+        network_info = get_network_info_for_port(context, port_id, network_id)
 
         if not network_info:
             LOG.warning("No network information for net %s", network_id)
@@ -317,7 +351,7 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
                 # Format BGPVPN before sending notification
                 self.agent_rpc.delete_bgpvpn(
                     context,
-                    self._format_bgpvpn(bgpvpn, net_id))
+                    self._format_bgpvpn(context, bgpvpn, net_id))
 
     def update_bgpvpn_precommit(self, context, old_bgpvpn, bgpvpn):
         self._common_precommit_checks(bgpvpn)
@@ -336,10 +370,7 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
                     (key in changed_keys for key in ('route_targets',
                                                      'import_targets',
                                                      'export_targets'))):
-                    self.agent_rpc.update_bgpvpn(
-                        context,
-                        self._format_bgpvpn(bgpvpn, net_id)
-                    )
+                    self._update_bgpvpn_for_network(context, net_id, bgpvpn)
 
     def create_net_assoc_precommit(self, context, net_assoc):
         if network_is_external(context, net_assoc['network_id']):
@@ -348,36 +379,21 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
     def create_net_assoc_postcommit(self, context, net_assoc):
         if get_network_ports(context, net_assoc['network_id']):
             bgpvpn = self.get_bgpvpn(context, net_assoc['bgpvpn_id'])
-            formated_bgpvpn = self._format_bgpvpn(bgpvpn,
-                                                  net_assoc['network_id'])
-            self.agent_rpc.update_bgpvpn(
-                context,
-                formated_bgpvpn)
+            self._update_bgpvpn_for_network(context,
+                                            net_assoc['network_id'], bgpvpn)
+
+    def _update_bgpvpn_for_network(self, context, net_id, bgpvpn):
+        formated_bgpvpn = self._format_bgpvpn(context, bgpvpn, net_id)
+        self.agent_rpc.update_bgpvpn(context,
+                                     formated_bgpvpn)
 
     def delete_net_assoc_postcommit(self, context, net_assoc):
         if get_network_ports(context, net_assoc['network_id']):
             bgpvpn = self.get_bgpvpn(context, net_assoc['bgpvpn_id'])
-            formated_bgpvpn = self._format_bgpvpn(bgpvpn,
+            formated_bgpvpn = self._format_bgpvpn(context, bgpvpn,
                                                   net_assoc['network_id'])
             self.agent_rpc.delete_bgpvpn(context, formated_bgpvpn)
 
-    def _get_port(self, context, port_id):
-        # we need to look in the db for two reasons:
-        # - the port dict, as provided by the registry callback has no
-        #   port binding information
-        # - for some notifications, the current port is not included
-
-        _core_plugin = manager.NeutronManager.get_plugin()
-        # TODO(tmorin): should not need an admin context
-        return _core_plugin.get_port(n_context.get_admin_context(), port_id)
-
-    def _get_port_host(self, context, port):
-        if portbindings.HOST_ID not in port:
-            raise Exception("cannot determine host_id for port %s, "
-                            "aborting BGPVPN update", port['id'])
-
-        return port.get(portbindings.HOST_ID)
-
     def _ignore_port(self, context, port):
         if (port['device_owner'].startswith(const.DEVICE_OWNER_NETWORK_PREFIX)
                 and not port['device_owner'] in
@@ -394,26 +410,21 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
 
         return False
 
-    def notify_port_updated(self, context, port_id, original_port):
-        LOG.info("notify_port_updated on port %s", port_id)
-
-        port = self._get_port(context, port_id)
-
-        port_bgpvpn_info = {'id': port['id'],
-                            'network_id': port['network_id']}
+    @log_helpers.log_method_call
+    def notify_port_updated(self, context, port, original_port):
 
         if self._ignore_port(context, port):
             return
 
-        agent_host = self._get_port_host(context, port)
+        agent_host = port[portbindings.HOST_ID]
 
-        original_port_status = "not present in notification"
-        if original_port:
-            original_port_status = original_port['status']
+        port_bgpvpn_info = {'id': port['id'],
+                            'network_id': port['network_id']}
 
         if (port['status'] == const.PORT_STATUS_ACTIVE and
-                original_port_status != const.PORT_STATUS_ACTIVE):
-            LOG.info("notify_port_updated, port became ACTIVE")
+                original_port['status'] != const.PORT_STATUS_ACTIVE):
+            LOG.debug("notify_port_updated, port became ACTIVE")
+
             bgpvpn_network_info = (
                 self._retrieve_bgpvpn_network_info_for_port(context, port)
             )
@@ -432,31 +443,26 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
                 pass
 
         elif (port['status'] == const.PORT_STATUS_DOWN and
-                original_port_status != const.PORT_STATUS_DOWN):
-            LOG.info("notify_port_updated, port became DOWN")
+                original_port['status'] != const.PORT_STATUS_DOWN):
+            LOG.debug("notify_port_updated, port became DOWN")
             self.agent_rpc.detach_port_from_bgpvpn(context,
                                                    port_bgpvpn_info,
                                                    agent_host)
         else:
             LOG.debug("new port status is %s, origin status was %s,"
-                      " => no action", port['status'], original_port_status)
-
-    def notify_port_deleted(self, context, port_id):
-        LOG.info("notify_port_deleted on port %s ", port_id)
-
-        port = self._get_port(context, port_id)
+                      " => no action", port['status'], original_port['status'])
 
+    @log_helpers.log_method_call
+    def notify_port_deleted(self, context, port):
         port_bgpvpn_info = {'id': port['id'],
                             'network_id': port['network_id']}
 
         if self._ignore_port(context, port):
             return
 
-        agent_host = self._get_port_host(context, port)
-
         self.agent_rpc.detach_port_from_bgpvpn(context,
                                                port_bgpvpn_info,
-                                               agent_host)
+                                               port[portbindings.HOST_ID])
 
     def create_router_assoc_postcommit(self, context, router_assoc):
         ports = get_router_ports(context, router_assoc['router_id'])
@@ -476,103 +482,84 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
                              'bgpvpn_id': router_assoc['bgpvpn_id']}
                 self.delete_net_assoc_postcommit(context, net_assoc)
 
-    def router_interface_added(self, context, port):
-        net_assocs = get_network_bgpvpn_assocs(context, port['network_id'])
-        router_assocs = get_router_bgpvpn_assocs(context, port['device_id'])
-        if net_assocs and router_assocs:
-            msg = 'Can not add router interface because both router %(rtr)s ' \
-                  'and network %(net)s have bgpvpn association(s)'
-            LOG.error(msg % {'rtr': port['device_id'],
-                             'net': port['network_id']})
-            return
+    @log_helpers.log_method_call
+    def notify_router_interface_created(self, context, router_id, net_id):
+        net_assocs = get_network_bgpvpn_assocs(context, net_id)
+        router_assocs = get_router_bgpvpn_assocs(context, router_id)
+
+        # if this router_interface is on a network bound to a BGPVPN,
+        # or if this router is bound to a BGPVPN,
+        # then we need to send and update for this network, including
+        # the gateway_mac
+        if net_assocs or router_assocs:
+            for bgpvpn in self._bgpvpns_for_network(context, net_id):
+                self._update_bgpvpn_for_network(context, net_id, bgpvpn)
+
         for router_assoc in router_assocs:
-            net_assoc = {'network_id': port['network_id'],
+            net_assoc = {'network_id': net_id,
                          'bgpvpn_id': router_assoc['bgpvpn_id']}
             self.create_net_assoc_postcommit(context, net_assoc)
 
-    def router_interface_removed(self, context, port):
-        for router_assoc in get_router_bgpvpn_assocs(context,
-                                                     port['device_id']):
-            net_assoc = {'network_id': port['network_id'],
+    @log_helpers.log_method_call
+    def notify_router_interface_deleted(self, context, router_id, net_id):
+        net_assocs = get_network_bgpvpn_assocs(context, net_id)
+        router_assocs = get_router_bgpvpn_assocs(context, router_id)
+
+        if net_assocs or router_assocs:
+            for bgpvpn in self._bgpvpns_for_network(context, net_id):
+                self._update_bgpvpn_for_network(context, net_id, bgpvpn)
+
+        for router_assoc in router_assocs:
+            net_assoc = {'network_id': net_id,
                          'bgpvpn_id': router_assoc['bgpvpn_id']}
             self.delete_net_assoc_postcommit(context, net_assoc)
 
     @log_helpers.log_method_call
     def registry_port_updated(self, resource, event, trigger, **kwargs):
         try:
-            context = kwargs.get('context')
-            port = kwargs.get('port')
-            original_port = kwargs.get('original_port')
-            # In notifications coming from ml2/plugin.py update_port
-            # it is possible that 'port' may be None, in this
-            # case we will use original_port
-            if port is None:
-                port = original_port
+            context = kwargs['context']
+            port = kwargs['port']
+            original_port = kwargs['original_port']
 
-            rtr_itf_added = self._is_router_intf_added(original_port, port)
-            rtr_itf_removed = self._is_router_intf_added(port, original_port)
-
-            if rtr_itf_added:
-                self.router_interface_added(context, port)
-            elif rtr_itf_removed:
-                self.router_interface_removed(context, original_port)
-            else:
-                self.notify_port_updated(context, port['id'], original_port)
+            self.notify_port_updated(context, port, original_port)
         except Exception as e:
-            LOG.exception(_LE("Error during notification processing "
-                              "%(resource)s %(event)s, %(trigger)s, "
-                              "%(kwargs)s: %(exc)s"),
-                          {'trigger': trigger,
-                           'resource': resource,
-                           'event': event,
-                           'kwargs': kwargs,
-                           'exc': e})
-
-    def _is_router_intf_added(self, original_port, port):
-        if not (original_port and port):
-            # This happens when the ml2 rpc.py sends a port update
-            # notification. original port will be None. However, this rpc
-            # notification will never be from a port device_id/device_owner
-            # update, so return False
-            return False
-        return (original_port['device_owner']
-                != const.DEVICE_OWNER_ROUTER_INTF
-                == port['device_owner'])
+            _log_callback_processing_exception(resource, event, trigger,
+                                               kwargs, e)
 
     @log_helpers.log_method_call
     def registry_port_deleted(self, resource, event, trigger, **kwargs):
         try:
-            context = kwargs.get('context')
-            port_id = kwargs.get('port_id')
-            port = self._get_port(context, port_id)
+            context = kwargs['context']
+            port = kwargs['port']
 
-            if port['device_owner'] == const.DEVICE_OWNER_ROUTER_INTF:
-                self.router_interface_removed(context, port)
-            else:
-                self.notify_port_deleted(context, port_id)
+            self.notify_port_deleted(context, port)
         except Exception as e:
-            LOG.exception(_LE("Error during notification processing "
-                              "%(resource)s %(event)s, %(trigger)s, "
-                              "%(kwargs)s: %(exc)s"),
-                          {'trigger': trigger,
-                           'resource': resource,
-                           'event': event,
-                           'kwargs': kwargs,
-                           'exc': e})
+            _log_callback_processing_exception(resource, event, trigger,
+                                               kwargs, e)
 
     @log_helpers.log_method_call
-    def registry_port_created(self, resource, event, trigger, **kwargs):
+    def registry_router_interface_created(self, resource, event, trigger,
+                                          **kwargs):
         try:
-            context = kwargs.get('context')
-            port = kwargs.get('port')
-            if port and port['device_owner'] == const.DEVICE_OWNER_ROUTER_INTF:
-                self.router_interface_added(context, port)
+            context = kwargs['context']
+            router_id = kwargs['router_id']
+            net_id = kwargs['port']['network_id']
+            self.notify_router_interface_created(context, router_id, net_id)
         except Exception as e:
-            LOG.exception(_LE("Error during notification processing "
-                              "%(resource)s %(event)s, %(trigger)s, "
-                              "%(kwargs)s: %(exc)s"),
-                          {'trigger': trigger,
-                           'resource': resource,
-                           'event': event,
-                           'kwargs': kwargs,
-                           'exc': e})
+            _log_callback_processing_exception(resource, event, trigger,
+                                               kwargs, e)
+
+    @log_helpers.log_method_call
+    def registry_router_interface_deleted(self, resource, event, trigger,
+                                          **kwargs):
+        try:
+            context = kwargs['context']
+            # for router_interface after_delete, in stable/newton, the
+            # callback does not include the router_id directly, but we find
+            # it in the port device_id
+            router_id = kwargs['port']['device_id']
+            net_id = kwargs['port']['network_id']
+            self.notify_router_interface_deleted(context, router_id, net_id)
+        except Exception as e:
+            _log_callback_processing_exception(resource, event, trigger,
+                                               kwargs, e)
