@@ -220,10 +220,10 @@ class OpenContrailBGPVPNDriver(driver_api.BGPVPNDriverBase):
         try:
             bgpvpn = json.loads(oc_client.kv_store('RETRIEVE', key=id))
         except (oc_exc.OpenContrailAPINotFound, ValueError):
-            raise bgpvpn.BGPVPNNotFound(id=id)
+            raise bgpvpn_ext.BGPVPNNotFound(id=id)
 
         if (not isinstance(bgpvpn, dict) or 'bgpvpn' not in bgpvpn):
-            raise bgpvpn.BGPVPNNotFound(id=id)
+            raise bgpvpn_ext.BGPVPNNotFound(id=id)
 
         bgpvpn = bgpvpn['bgpvpn']
         if not fields or 'networks' in fields:
@@ -330,7 +330,7 @@ class OpenContrailBGPVPNDriver(driver_api.BGPVPNDriverBase):
                           'export_targets']
             bgpvpn = self.get_bgpvpn(context, net_assoc['bgpvpn_id'],
                                      fields=get_fields)
-        except bgpvpn.BGPVPNNotFound:
+        except bgpvpn_ext.BGPVPNNotFound:
             raise bgpvpn_ext.BGPVPNNetAssocNotFound(id=assoc_id,
                                                     bgpvpn_id=bgpvpn_id)
 
@@ -406,12 +406,6 @@ class OpenContrailBGPVPNDriver(driver_api.BGPVPNDriverBase):
 
         return net_assoc
 
-    def find_bgpvpns_for_network(self, context, network_id, bgpvpn_type=None):
-        # TODO(matrohon) : get bgpvpns from the kv store
-        raise bgpvpn_ext.BGPVPNFindFromNetNotSupported(
-            driver=OPENCONTRAIL_BGPVPN_DRIVER_NAME,
-            net_id=network_id)
-
     def create_router_assoc(self, context, bgpvpn_id, router_association):
         raise bgpvpn_ext.BGPVPNRouterAssociationNotSupported(
             driver=OPENCONTRAIL_BGPVPN_DRIVER_NAME)
@@ -427,7 +421,3 @@ class OpenContrailBGPVPNDriver(driver_api.BGPVPNDriverBase):
     def delete_router_assoc(self, context, assoc_id, bgpvpn_id):
         raise bgpvpn_ext.BGPVPNRouterAssociationNotSupported(
             driver=OPENCONTRAIL_BGPVPN_DRIVER_NAME)
-
-    def find_bgpvpns_for_router(self, context, router_id):
-        raise bgpvpn_ext.BGPVPNRouterAssociationNotSupported(
-            driver=OPENCONTRAIL_BGPVPN_DRIVER_NAME)
