@@ -22,6 +22,8 @@ from oslo_utils import excutils
 
 from networking_odl.common import client as odl_client
 
+from networking_bgpvpn._i18n import _LW
+
 from networking_bgpvpn.neutron.extensions import bgpvpn as bgpvpn_ext
 from networking_bgpvpn.neutron.services.service_drivers import driver_api
 
@@ -41,23 +43,21 @@ class OpenDaylightBgpvpnDriver(driver_api.BGPVPNDriver):
     """
 
     def __init__(self, service_plugin):
-        LOG.debug("Initializing OpenDaylight BGPVPN driver")
+        LOG.warning(_LW("This OpenDaylight BGPVPN driver has been deprecated"
+                        "and will be removed. Switch to new v2 Driver: "
+                        "BGPVPN:OpenDaylight:networking_odl.bgpvpn.odl_v2."
+                        "OpenDaylightBgpvpnDriver:default"))
         super(OpenDaylightBgpvpnDriver, self).__init__(service_plugin)
         self.service_plugin = service_plugin
 
         self.client = odl_client.OpenDaylightRestClient.create_client()
 
-    def _scrub_rd_list(self, bgpvpn):
-        if len(bgpvpn['route_distinguishers']) > 1:
-            bgpvpn['route_distinguishers'] = bgpvpn['route_distinguishers'][0]
-
     def create_bgpvpn_precommit(self, context, bgpvpn):
         pass
 
     def create_bgpvpn_postcommit(self, context, bgpvpn):
         url = BGPVPNS
         try:
-            self._scrub_rd_list(bgpvpn)
             self.client.sendjson('post', url, {BGPVPNS[:-1]: bgpvpn})
         except requests.exceptions.RequestException:
             with excutils.save_and_reraise_exception():
@@ -74,8 +74,12 @@ class OpenDaylightBgpvpnDriver(driver_api.BGPVPNDriver):
         self.client.sendjson('put', url, {BGPVPNS[:-1]: bgpvpn})
 
     def create_net_assoc_precommit(self, context, net_assoc):
-        bgpvpns = self.bgpvpn_db.find_bgpvpns_for_network(
-            context, net_assoc['network_id'])
+        bgpvpns = self.bgpvpn_db.get_bgpvpns(
+            context,
+            filters={
+                'networks': [net_assoc['network_id']],
+            },
+        )
         if len(bgpvpns) > 1:
             raise bgpvpn_ext.BGPVPNNetworkAssocExistsAnotherBgpvpn(
                 driver=OPENDAYLIGHT_BGPVPN_DRIVER_NAME,
@@ -85,7 +89,6 @@ class OpenDaylightBgpvpnDriver(driver_api.BGPVPNDriver):
     def create_net_assoc_postcommit(self, context, net_assoc):
         bgpvpn = self.get_bgpvpn(context, net_assoc['bgpvpn_id'])
         url = BGPVPNS + '/' + bgpvpn['id']
-        self._scrub_rd_list(bgpvpn)
         try:
             self.client.sendjson('put', url, {BGPVPNS[:-1]: bgpvpn})
         except requests.exceptions.RequestException:
@@ -111,7 +114,6 @@ class OpenDaylightBgpvpnDriver(driver_api.BGPVPNDriver):
     def create_router_assoc_postcommit(self, context, router_assoc):
         bgpvpn = self.get_bgpvpn(context, router_assoc['bgpvpn_id'])
         url = BGPVPNS + '/' + bgpvpn['id']
-        self._scrub_rd_list(bgpvpn)
         try:
             self.client.sendjson('put', url, {BGPVPNS[:-1]: bgpvpn})
         except requests.exceptions.RequestException:
