@@ -18,10 +18,9 @@ import copy
 import mock
 import webob.exc
 
+from neutron_lib.plugins import directory
 from oslo_utils import uuidutils
 
-from neutron import manager
-
 from neutron.api import extensions as api_extensions
 from neutron.db import servicetype_db as sdb
 from neutron import extensions as n_extensions
@@ -88,8 +87,7 @@ class BgpvpnTestCaseMixin(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
 
         # find the BGPVPN plugin that was instantiated by the
         # extension manager:
-        self.bgpvpn_plugin = (manager.NeutronManager.get_service_plugins()
-                              [constants.BGPVPN])
+        self.bgpvpn_plugin = directory.get_plugin(constants.BGPVPN)
 
         self.bgpvpn_data = {'bgpvpn': {'name': 'bgpvpn1',
                                        'type': 'l3',
@@ -102,14 +100,15 @@ class BgpvpnTestCaseMixin(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
 
     @contextlib.contextmanager
     def bgpvpn(self, do_delete=True, **kwargs):
+        req_data = copy.deepcopy(self.bgpvpn_data)
+
         fmt = 'json'
         if kwargs.get('data'):
-            bgpvpn_data = kwargs.get('data')
+            req_data = kwargs.get('data')
         else:
-            bgpvpn_data = copy.copy(self.bgpvpn_data)
-            bgpvpn_data['bgpvpn'].update(kwargs)
+            req_data['bgpvpn'].update(kwargs)
         bgpvpn_req = self.new_create_request(
-            'bgpvpn/bgpvpns', bgpvpn_data, fmt=fmt)
+            'bgpvpn/bgpvpns', req_data, fmt=fmt)
         res = bgpvpn_req.get_response(self.ext_api)
         if res.status_int >= 400:
             raise webob.exc.HTTPClientError(code=res.status_int)
@@ -181,16 +180,16 @@ class TestBGPVPNServicePlugin(BgpvpnTestCaseMixin):
 
     @mock.patch.object(plugin.BGPVPNPlugin, '_validate_network')
     def test_bgpvpn_net_assoc_create(self, mock_validate):
-        with self.network() as net:
-            net_id = net['network']['id']
-            with self.bgpvpn() as bgpvpn:
-                id = bgpvpn['bgpvpn']['id']
-                mock_validate.return_value = net['network']
-                with self.assoc_net(id, net_id):
-                    net_body = {'network_id': net['network']['id'],
-                                'tenant_id': self._tenant_id}
-                    mock_validate.assert_called_once_with(mock.ANY,
-                                                          net_body)
+        with self.network() as net, \
+                self.bgpvpn() as bgpvpn, \
+                mock.patch.object(
+                    self.bgpvpn_plugin,
+                    '_validate_network',
+                    return_value=net['network']) as mock_validate, \
+                self.assoc_net(bgpvpn['bgpvpn']['id'],
+                               net['network']['id']):
+                    mock_validate.assert_called_once_with(
+                        mock.ANY, net['network']['id'])
 
     def test_associate_empty_network(self):
         with self.bgpvpn() as bgpvpn:
@@ -252,18 +251,17 @@ class TestBGPVPNServicePlugin(BgpvpnTestCaseMixin):
                 res = bgpvpn_net_req.get_response(self.ext_api)
                 self.assertEqual(res.status_int, webob.exc.HTTPForbidden.code)
 
-    @mock.patch.object(plugin.BGPVPNPlugin, '_validate_router')
-    def test_bgpvpn_router_assoc_create(self, mock_validate):
-        with self.router(tenant_id=self._tenant_id) as router:
-            router_id = router['router']['id']
-            with self.bgpvpn() as bgpvpn:
-                id = bgpvpn['bgpvpn']['id']
-                mock_validate.return_value = router['router']
-                with self.assoc_router(id, router_id):
-                    router_body = {'router_id': router['router']['id'],
-                                   'tenant_id': self._tenant_id}
-                    mock_validate.assert_called_once_with(mock.ANY,
-                                                          router_body)
+    def test_bgpvpn_router_assoc_create(self):
+        with self.router(tenant_id=self._tenant_id) as router,\
+            self.bgpvpn() as bgpvpn, \
+            mock.patch.object(
+                self.bgpvpn_plugin,
+                '_validate_router',
+                return_value=router['router']) as mock_validate, \
+            self.assoc_router(bgpvpn['bgpvpn']['id'],
+                              router['router']['id']):
+                mock_validate.assert_called_once_with(
+                    mock.ANY, router['router']['id'])
 
     def test_associate_empty_router(self):
         with self.bgpvpn() as bgpvpn:
@@ -437,6 +435,45 @@ class TestBGPVPNServicePlugin(BgpvpnTestCaseMixin):
             res = bgpvpn_router_req.get_response(self.ext_api)
             self.assertEqual(res.status_int, webob.exc.HTTPBadRequest.code)
 
+    def test_attach_subnet_to_router_both_attached_to_bgpvpn(self):
+        with self.network() as net,\
+                self.bgpvpn() as bgpvpn,\
+                self.router(tenant_id=self._tenant_id) as router,\
+                self.subnet(network={'network': net['network']}) as subnet,\
+                self.assoc_net(bgpvpn['bgpvpn']['id'], net['network']['id']),\
+                self.assoc_router(bgpvpn['bgpvpn']['id'],
+                                  router['router']['id']):
+            # Attach subnet to router
+            data = {"subnet_id": subnet['subnet']['id']}
+            bgpvpn_rtr_intf_req = self.new_update_request(
+                'routers',
+                data=data,
+                fmt=self.fmt,
+                id=router['router']['id'],
+                subresource='add_router_interface')
+            res = bgpvpn_rtr_intf_req.get_response(self.ext_api)
+            self.assertEqual(res.status_int, webob.exc.HTTPConflict.code)
+
+    def test_attach_port_to_router_both_attached_to_bgpvpn(self):
+        with self.network() as net,\
+                self.bgpvpn() as bgpvpn,\
+                self.router(tenant_id=self._tenant_id) as router,\
+                self.subnet(network={'network': net['network']}) as subnet,\
+                self.port(subnet={'subnet': subnet['subnet']}) as port,\
+                self.assoc_net(bgpvpn['bgpvpn']['id'], net['network']['id']),\
+                self.assoc_router(bgpvpn['bgpvpn']['id'],
+                                  router['router']['id']):
+            # Attach subnet to router
+            data = {"port_id": port['port']['id']}
+            bgpvpn_rtr_intf_req = self.new_update_request(
+                'routers',
+                data=data,
+                fmt=self.fmt,
+                id=router['router']['id'],
+                subresource='add_router_interface')
+            res = bgpvpn_rtr_intf_req.get_response(self.ext_api)
+            self.assertEqual(res.status_int, webob.exc.HTTPConflict.code)
+
 
 class TestBGPVPNServiceDriverDB(BgpvpnTestCaseMixin):
 
@@ -457,8 +494,10 @@ class TestBGPVPNServiceDriverDB(BgpvpnTestCaseMixin):
                            mock_create_postcommit):
         mock_create_db.return_value = self.converted_data['bgpvpn']
         with self.bgpvpn(do_delete=False):
-            mock_create_db.assert_called_once_with(
-                mock.ANY, self.converted_data['bgpvpn'])
+            self.assertTrue(mock_create_db.called)
+            self.assertDictSupersetOf(
+                self.converted_data['bgpvpn'],
+                mock_create_db.call_args[0][1])
             mock_create_precommit.assert_called_once_with(
                 mock.ANY, self.converted_data['bgpvpn'])
             mock_create_postcommit.assert_called_once_with(
@@ -539,18 +578,19 @@ class TestBGPVPNServiceDriverDB(BgpvpnTestCaseMixin):
             old_bgpvpn['id'] = bgpvpn['bgpvpn']['id']
             old_bgpvpn['networks'] = []
             old_bgpvpn['routers'] = []
+            old_bgpvpn['project_id'] = old_bgpvpn['tenant_id']
             new_bgpvpn = copy.copy(old_bgpvpn)
             new_bgpvpn['name'] = 'foo'
 
             mock_update_db.return_value = new_bgpvpn
 
-            new_data = {"bgpvpn": {"name": "foo"}}
+            data = {"bgpvpn": {"name": new_bgpvpn['name']}}
             self._update('bgpvpn/bgpvpns',
                          bgpvpn['bgpvpn']['id'],
-                         new_data)
+                         data)
 
             mock_update_db.assert_called_once_with(
-                mock.ANY, bgpvpn['bgpvpn']['id'], new_data['bgpvpn'])
+                mock.ANY, bgpvpn['bgpvpn']['id'], data['bgpvpn'])
             mock_update_precommit.assert_called_once_with(
                 mock.ANY, old_bgpvpn, new_bgpvpn)
             mock_update_postcommit.assert_called_once_with(
@@ -592,8 +632,12 @@ class TestBGPVPNServiceDriverDB(BgpvpnTestCaseMixin):
                 mock_db_create_assoc.return_value = net_assoc_dict
                 with self.assoc_net(bgpvpn_id, net_id=net_id,
                                     do_disassociate=False):
-                    mock_db_create_assoc.assert_called_once_with(
-                        mock.ANY, bgpvpn_id, data)
+                    self.assertTrue(mock_db_create_assoc.called)
+                    self.assertEqual(
+                        bgpvpn_id, mock_db_create_assoc.call_args[0][1])
+                    self.assertDictSupersetOf(
+                        data,
+                        mock_db_create_assoc.call_args[0][2])
                     mock_pre_commit.assert_called_once_with(mock.ANY,
                                                             net_assoc_dict)
                     mock_post_commit.assert_called_once_with(mock.ANY,
@@ -694,8 +738,12 @@ class TestBGPVPNServiceDriverDB(BgpvpnTestCaseMixin):
             mock_db_create_assoc.return_value = router_assoc_dict
             with self.assoc_router(bgpvpn_id, router_id=router_id,
                                    do_disassociate=False):
-                mock_db_create_assoc.assert_called_once_with(
-                    mock.ANY, bgpvpn_id, data)
+                self.assertTrue(mock_db_create_assoc.called)
+                self.assertEqual(
+                    bgpvpn_id, mock_db_create_assoc.call_args[0][1])
+                self.assertDictSupersetOf(
+                    data,
+                    mock_db_create_assoc.call_args[0][2])
                 mock_pre_commit.assert_called_once_with(mock.ANY,
                                                         router_assoc_dict)
                 mock_post_commit.assert_called_once_with(mock.ANY,
