@@ -14,15 +14,12 @@
 #    under the License.
 
 from networking_bgpvpn_tempest.services.bgpvpn import bgpvpn_client
-from oslo_log import log as logging
 import tempest.api.network.base as test
 from tempest.common.utils import data_utils
 from tempest import config
 
 CONF = config.CONF
 
-LOG = logging.getLogger(__name__)
-
 
 class BaseBgpvpnTest(test.BaseNetworkTest):
     """Base class for the Bgpvpn tests that use the Tempest Neutron REST client
